# -*- coding: utf-8 -*-

import requests as requests
from requests import (
    utils, packages, Request, Response,
    PreparedRequest, session, Session, codes
)
from requests.exceptions import (
    RequestException, Timeout, URLRequired,
    TooManyRedirects, HTTPError, ConnectionError,
    ConnectTimeout, ReadTimeout
)

from .api import request, get, options, head, post, put, patch, delete
from .model import SafeResponse, StatusFilter



# -*- coding: utf-8 -*-

from time import sleep
from requests import Session
from .model import SafeResponse, StatusFilter


def request(method, url, session=None, retry=5, wait=1, status=None, **kwargs):
    session: Session
    if status is None:
        status = []
    elif type(status) == int:
        status = [status]
    if type(status) == list:
        status = StatusFilter(status)
    if session is None:
        session = Session()

    try_count = 0

    while try_count < retry:
        try_count += 1
        if try_count != 1:
            sleep(wait)
        try:
            res = session.request(method=method, url=url, **kwargs)
            session.close()
            if status.compile(res.status_code):
                return SafeResponse(success=True, response=res, session=session)
        except Exception:
            continue
    return SafeResponse(success=False)


def get(url, params=None, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a GET request.

    :param url: URL for the new :class:`Request` object.
    :param params: (optional) Dictionary, list of tuples or bytes to send
        in the query string for the :class:`Request`
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    kwargs.setdefault('allow_redirects', True)
    return request('get', url, params=params, session=session, retry=retry, wait=wait, status=status, **kwargs)


def options(url, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends an OPTIONS request.

    :param url: URL for the new :class:`Request` object.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    kwargs.setdefault('allow_redirects', True)
    return request('options', url, session=session, retry=retry, wait=wait, status=status, **kwargs)


def head(url, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a HEAD request.

    :param url: URL for the new :class:`Request` object.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes. If
        `allow_redirects` is not provided, it will be set to `False` (as
        opposed to the default :meth:`request` behavior).
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    kwargs.setdefault('allow_redirects', False)
    return request('head', url, session=session, retry=retry, wait=wait, status=status, **kwargs)


def post(url, data=None, json=None, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a POST request.

    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    return request('post', url, data=data, json=json, session=session,
                   retry=retry, wait=wait, status=status, **kwargs)


def put(url, data=None, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a PUT request.

    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    return request('put', url, data=data, session=session, retry=retry, wait=wait, status=status, **kwargs)


def patch(url, data=None, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a PATCH request.

    :param url: URL for the new :class:`Request` object.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) json data to send in the body of the :class:`Request`.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    return request('patch', url, data=data, session=session, retry=retry, wait=wait, status=status, **kwargs)


def delete(url, session=None, retry=5, wait=1, status=None, **kwargs):
    r"""Sends a DELETE request.

    :param url: URL for the new :class:`Request` object.
    :param session: request with session
    :type session: requests.Session
    :param retry: Max retry count.
    :param wait: Time(seconds) between retry
    :param status: Response status code filter
    :param \*\*kwargs: Optional arguments that ``request`` takes.
    :return: :class:`Response <Response>` object
    :rtype: reqWrapper.SafeResponse
    """

    return request('delete', url, session=session, retry=retry, wait=wait, status=status, **kwargs)



from setuptools import setup

setup(
    name='reqWrapper',
    version='0.2.0',
    packages=['reqWrapper'],
    url='https://github.com/box-archived/reqWrap',
    license='Apache License 2.0',
    author='box-archived',
    author_email='box.cassette@gmail.com',
    description='Wrapper of requests with retry options',
    install_requires=['requests>=2.*']
)



# -*- coding: utf-8 -*-

from requests import Response, Session


class SafeResponse(object):
    def __init__(self, success, response=None, session=None):
        self.__success: bool = success
        self.__response: Response = response
        self.__session: Session = session

    def __repr__(self):
        if self.success:
            status = "Success: %s" % self.response.status_code
        else:
            status = "Failed"
        return '<SafeResponse [%s]>' % status

    @property
    def success(self):
        return self.__success

    @property
    def response(self):
        if self.success:
            return self.__response
        else:
            return None

    @property
    def status_code(self):
        if self.success:
            return self.__response.status_code
        else:
            return None

    @property
    def session(self):
        if self.success:
            return self.__session
        else:
            return None


class StatusFilter(set):

    def interpret_options(self):
        """ Interpret option set
        1. Parse :class `range`
        2. Make positive list
            range(100,600) if len*(positive) == 0
        :return: positive-set, negative-set
        """
        positive = set()
        negative = set()
        for item in self:

            # Case <range item>
            if type(item) == range:
                if item.start + item.stop > 0:
                    positive |= set(item)
                else:
                    negative |= set(item)

            # Case <int item>
            elif type(item) == int:
                if item >= 100:
                    positive.add(item)
                elif item <= -100:
                    negative.add(item)

            else:
                raise ValueError("Interpret Failed")

        return positive, negative

    def pass_list(self):
        r""" Make include list from options

        :return: list `include list`
        :rtype: list
        """

        positive: set
        negative: set
        positive, negative = self.interpret_options()

        # If positive set is empty, make all-pass-set of status code
        if len(positive) == 0:
            positive = set(range(100, 600))

        # Exclude negative items
        for item in negative:
            while -item in positive:
                positive.remove(-item)

        return list(positive)

    def compile(self, status_code):
        r""" Test :param `status_code` is pass

        :param status_code:
        :return: success or fail
        :rtype: bool
        """
        if status_code in self.pass_list():
            return True
        else:
            return False
